#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers
from zunclient.tests.unit import utils
from zunclient.v1 import availability_zones as az

AZ1 = {'availability_zone': 'fake-az-1'}

AZ2 = {'availability_zone': 'fake-az-2'}

fake_responses = {
    '/v1/availability_zones':
    {
        'GET': (
            {},
            {'availability_zones': [AZ1, AZ2]},
        ),
    },
}


class AZManagerTest(testtools.TestCase):

    def setUp(self):
        super(AZManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = az.AvailabilityZoneManager(self.api)

    def test_availability_zones_list(self):
        zones = self.mgr.list()
        expect = [
            ('GET', '/v1/availability_zones', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(zones, matchers.HasLength(2))
